/**
 * \file gen_array3.hpp
 *
 * Generikus tmb NHF plda feladatnak bemutatshoz.
 * Ksz (vgleges) feladat.
 *
 * Feladat:
 *   Fix mret Generikus tmb megvalstsa.
 */

#include <sstream>          /// csak a hibajelzs miatt kell

/// Generikus tmb osztly statikus mrettel
/// @param T - adattpus
/// @param s - mret, default: 50
/// @param E - hibaosztly (indexelsi hiba esetn hvdik) defalul: std::range_error
template <class T, int s = 50, typename E = std::out_of_range>
class Array {               // osztlysablon
    T t[s];                 /// adattag, template paramter a mrete
public:
    /// Indexels.
    /// @param i - index
    /// @return - referencia az adott index elemre
    /// @return - hibs indexrtk esetn E() kivtelt dob
    T& operator[](int i);

    /// Indexels konstans fggvnye.
    /// @param i - index
    /// @return - referencia az adott index elemre
    /// @return - hibs indexrtk esetn E() kivtelt dob 
    const T& operator[](int i) const;
};	

/// index opertor megvalstsa
template <class T, int s, typename E>
T& Array<T, s, E>::operator [](int i) {
    if (i < 0 || i >= s) {
        // megprblunk sszerakni egy rtelmes hibajelzst
        // persze ez gy nagyon fapados s ronda
        std::stringstream err;
        err << " **Indexhiba** idx: " << i << " [0," << s-1 << "]";
        throw E(err.str()); // template paramterknt kapott osztly egy pldnyt dobjuk
    }
    return t[i];    // megfelel elem referencijval trnk vissza
}

/// konstans pldny index opertor megvalstsa
template <class T, int s, typename E>
const T& Array<T, s, E>::operator [](int i) const {
    if (i < 0 || i >= s) {
        // megprblunk sszerakni egy rtelmes hibajelzst
        // persze ez gy nagyon fapados s ronda
        std::stringstream err;
        err << " **Indexhiba** idx: " << i << " [0," << s-1 << "]";
        throw E(err.str()); // template paramterknt kapott osztly egy pldnyt dobjuk
    }
    return t[i];    // megfelel elem referencijval trnk vissza
}
